use minifb::{Scale, ScaleMode, Window, WindowOptions};
use serde::{Deserialize, Serialize};
#[cfg(feature = "term")]
use termion::{
    color::{self, Bg, Fg},
    cursor::Goto,
};

use crate::{
    cpu::CpuController,
    memory::{MemHandle, MemoryReader},
};
use std::{
    fs::File,
    io::{self, Read, Write},
    path::Path,
};

// const FG_COLOR: u32 = 0xFFCC00;
// const BG_COLOR: u32 = 0x110500;
const FG_COLOR: u32 = 0xFF00CCFF;
const BG_COLOR: u32 = 0xFF000511;
const WIDTH: usize = 512;
const HEIGHT: usize = 380;

// pub fn get_char_bin<P>(char_rom: Option<P>) -> [u8; 0x8000]
// where
//     P: AsRef<Path>,
// {
//     match char_rom {
//         Some(path) => {
//             let mut file = File::open(path).unwrap();
//             let mut bin = vec![0; 0x8000];
//             file.read_exact(&mut bin).unwrap();
//             // println!("reading char rom");
//             bin.try_into().unwrap()
//         }
//         None => *include_bytes!("./roms/cozette.rom"),
//     }
// }

#[cfg(not(feature = "term"))]
#[derive(Debug)]
pub struct Renderer {
    char_rom: [u8; 0x8000],
    memory: MemHandle,
    controller: CpuController,
}

#[cfg(not(feature = "term"))]
impl Renderer {
    // pub fn new<P>(controller: CpuController, memory: MemHandle, char_rom: Option<P>) -> Self
    // where
    //     P: AsRef<Path>,
    pub fn new(controller: CpuController, memory: MemHandle) -> Self {
        // let char_rom = get_char_bin(char_rom);
        let char_rom = *include_bytes!("./roms/cozette.rom");
        Self {
            memory,
            char_rom,
            controller,
        }
    }

    pub fn render(&self, window: &mut Window) {
        // the rest of this function is arcane wizardry
        // based on the specifics of george's weird
        // display and characters... don't fuck around w it
        let mut i = 0;
        let mut buffer = [0; 512 * 380];
        for char_row in 0..29 {
            for char_col in 0..64 {
                let ascii = self.read(0x6000 + i);
                i += 1;
                for row in 0..13 {
                    let byte = self.char_rom[ascii as usize + (row * 0x100)];
                    for bit_index in (0..8).rev() {
                        let buffer_index =
                            ((char_row) * 13 + (row)) * 512 + (char_col * 8 + bit_index);
                        if (byte << bit_index) & 0x80 == 0x80 {
                            buffer[buffer_index] = FG_COLOR;
                        } else {
                            buffer[buffer_index] = BG_COLOR;
                        }
                    }
                }
            }
        }
        self.controller.irq();
        window.update_with_buffer(&buffer, WIDTH, HEIGHT).unwrap();
    }
}

impl MemoryReader for Renderer {
    fn read(&self, address: u16) -> u8 {
        self.memory.read(address)
    }
}

#[cfg(feature = "term")]
pub struct Renderer {
    memory: MemHandle,
    controller: CpuController,
}

#[cfg(feature = "term")]
const ASCII_LOOPUP: [&str; 256] = [
    " ", "░", "▒", "▓", "♡", "♥", "⭐", "✭", "", "✦", "✨", "♀", "♂", "⚢", "⚣", "⚥", "♩", "♪",
    "♫", "♬", "ﱝ", "", "", "", "奄", "奔", "婢", "ﱜ", "ﱛ", "", "", "", " ", "!", "\"", "#",
    "$", "%", "&", "\'", "(", ")", "*", "+", ",", "-", ".", "/", "0", "1", "2", "3", "4", "5", "6",
    "7", "8", "9", ":", ";", "<", "=", ">", "?", "@", "A", "B", "C", "D", "E", "F", "G", "H", "I",
    "J", "K", "L", "M", "N", "O", "P", "Q", "R", "S", "T", "U", "V", "W", "X", "Y", "Z", "[", "\\",
    "]", "^", "_", "`", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o",
    "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", "{", "|", "}", "~", "─", "│", "┌", "└",
    "├", "┤", "┬", "┴", "┼", "╭", "╮", "╯", "╰", "╱", "╲", "╳", "═", "║", "╔", "╗", "╚", "╝", "╠",
    "╣", "╦", "╩", "╬", "", "", "", "", "", "", "", "", "", "", "", "", "", "▁", "▂",
    "▃", "▄", "▅", "▆", "▇", "█", "▉", "▊", "▋", "▌", "▍", "▎", "▏", "ʕ", "·", "ᴥ", "ʔ", "▖", "▗",
    "▘", "▙", "▚", "▛", "▜", "▝", "▞", "▟", "←", "↑", "→", "↓", "⭠", "⭡", "⭢", "⭣", "⮀", "⮁", "⮂",
    "⮃", "", "", "⏳", "", "", "", "", "", "", "", "", "", "", "", "🔮", "", "",
    "◇", "◈", "🌑", "🌒", "🌓", "🌔", "🌕", "🌖", "🌗", "🌘", "", "✔", "✘", "◆", "", "", "",
    "", "", "", "", "🎁", "", "", "", "", "⚐", "⚑", "", "", "",
];

#[cfg(feature = "term")]
impl Renderer {
    pub fn new(controller: CpuController, memory: MemHandle) -> Self {
        Self { controller, memory }
    }
    pub fn render(&self) {
        let mut stdout = io::stdout();
        let mut i = 0;
        for char_row in 0..29 {
            for char_col in 0..64 {
                let ascii = self.read(0x6000 + i);
                i += 1;
                let char = ASCII_LOOPUP[ascii as usize];
                let _ = write!(
                    // FG_COLOR = 0xFFCC00
                    // BG_COLOR = 0x110500
                    stdout,
                    "{}{}{}{char}",
                    Goto(char_col + 1, char_row + 1),
                    Fg(color::Rgb(0xFF, 0xCC, 0x00)),
                    Bg(color::Rgb(0x11, 0x05, 0x00))
                );
            }
        }
        self.controller.irq();
    }
}
